#include <Adafruit_LiquidCrystal.h>

String ssid     = "Simulator Wifi";  
String password = ""; 
String host     = "api.thingspeak.com";  
const int httpPort   = 80;
String apiKey  = "LOJARNSFQTSI90EP";  

const int MQ135_PIN = A0; // MQ-135 sensor (Gas Sensor)
const int TEMP_PIN  = A1; // Temperature Sensor
const int MOTOR_PIN = 5; // Air Purifier Motor

bool motorStatus = false; // Track motor state

// Initialize I2C LCD using Adafruit_LiquidCrystal (0x27 is default address)
Adafruit_LiquidCrystal lcd(0);

int setupESP8266(void) {
  Serial.begin(115200);
  Serial.println("AT");   
  delay(1000);
  if (!Serial.find("OK")) return 1;

  Serial.println("AT+CWJAP=\"" + ssid + "\",\"" + password + "\"");
  delay(5000); 
  if (!Serial.find("OK")) return 2;

  Serial.println("AT+CIPSTART=\"TCP\",\"" + host + "\"," + String(httpPort));
  delay(3000);
  if (!Serial.find("OK")) return 3;

  return 0;
}

// Function to calculate AQI from MQ-135 raw value
int calculateAQI(int gasValue) {
  // Ensure gasValue is within expected range
  gasValue = constrain(gasValue, 100, 1000); 

  // Map sensor readings correctly to 0-400 AQI range
  return map(gasValue, 50, 600, 0, 400);
}

// Function to get temperature (assuming LM35 sensor)
float readTemperature() {
  int rawValue = analogRead(TEMP_PIN);
  return (rawValue * 5.0 / 1023.0) * 100.0; // Convert to Celsius
}

// Turn purifier ON/OFF based on AQI
void controlPurifier(int aqi) {
  if (aqi > 150) { 
    digitalWrite(MOTOR_PIN, HIGH);
    motorStatus = true;
    Serial.println("Air Purifier ON 🚀");
  } else { 
    digitalWrite(MOTOR_PIN, LOW);
    motorStatus = false;
    Serial.println("Air Purifier OFF 🛑");
  }
}

// Update LCD with sensor values
void updateLCD(int gasValue, int aqi, float temp) {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("T:");
  lcd.print(temp);
  lcd.print("C  G:");
  lcd.print(gasValue);

  lcd.setCursor(0, 1);
  lcd.print("AQI:");
  lcd.print(aqi);
  lcd.setCursor(10, 1);
  lcd.print(motorStatus ? "ON" : "OFF");
}

void sendData(void) {
  int gasValue = analogRead(MQ135_PIN); // Read MQ-135 sensor
  int aqi = calculateAQI(gasValue); // Convert to AQI
  float temp = readTemperature(); // Read temperature sensor

  controlPurifier(aqi); // Check if purifier should turn ON/OFF
  updateLCD(gasValue, aqi, temp); // Show on LCD

  Serial.print("Temp: ");
  Serial.print(temp);
  Serial.print("C, Gas: ");
  Serial.print(gasValue);
  Serial.print(", AQI: ");
  Serial.print(aqi);
  Serial.print(", Motor: ");
  Serial.println(motorStatus ? "ON" : "OFF");

  // Send data to ThingSpeak
  String uri = "/update?api_key=" + apiKey + 
               "&field1=" + String(gasValue) + 
               "&field2=" + String(aqi) + 
               "&field3=" + String(motorStatus) +
               "&field4=" + String(temp);

  String httpPacket = "GET " + uri + " HTTP/1.1\r\nHost: " + host + "\r\nConnection: close\r\n\r\n";
  int length = httpPacket.length();

  Serial.print("AT+CIPSEND=");
  Serial.println(length);
  delay(1000);

  if (!Serial.find(">")) return;

  Serial.print(httpPacket);
  delay(1000);

  if (!Serial.find("SEND OK")) return;

  Serial.println("Data sent to ThingSpeak!");
}

void setup() {
  pinMode(MOTOR_PIN, OUTPUT);
  digitalWrite(MOTOR_PIN, LOW); // Ensure motor starts OFF

  lcd.begin(16, 2);
  lcd.setBacklight(HIGH);
  lcd.setCursor(0, 0);
  lcd.print("Initializing");

  if (setupESP8266() == 0) {
    Serial.println("ESP8266 Connected!");
    lcd.setCursor(0, 1);
    lcd.print("WiFi Connected!");
  } else {
    Serial.println("ESP8266 Connection Failed!");
    lcd.setCursor(0, 1);
    lcd.print("WiFi Failed!");
  }

  delay(2000);
}

void loop() {
  sendData();
  delay(15000); // ThingSpeak update every 15s
}

